import 'package:flutter/material.dart';
import 'package:sample/src/ui/pages/detail/detail_page.dart';
import 'package:sample/src/ui/pages/posts/posts_page.dart';
import 'package:sample/src/ui/pages/splash_screen.dart';

/// Standard routing class which determines which page should open depending on the provided route.
class Router {
  static Route<dynamic> generateRoute(RouteSettings settings) {
    final args = settings.arguments;

    switch (settings.name) {
      case PostsPage.routeName:
        return _createRoute((_) => PostsPage.create());
      case DetailPage.routeName:
        return _createRoute((_) => DetailPage.create(args));
      default:
        return _createRoute((_) => SplashScreenPage.create());
    }
  }

  static MaterialPageRoute _createRoute(WidgetBuilder builder) =>
      MaterialPageRoute(builder: builder);
}

/// Acts as an access point to navigate within the app without having to provide a context instance.
///
/// This way you can navigate within a BLoC as well.
class NavigationService {
  final GlobalKey<NavigatorState> navigatorKey = GlobalKey();

  Future<dynamic> pushNamed(String routeName, {Object args}) {
    return navigatorKey.currentState.pushNamed(routeName, arguments: args);
  }

  Future<dynamic> pushReplacementNamed(String routeName, {Object args}) {
    return navigatorKey.currentState
        .pushReplacementNamed(routeName, arguments: args);
  }
}
