import 'package:flutter/material.dart';
import 'package:mwb/bloc_provider.dart';
import 'package:sample/src/models/comment.dart';
import 'package:sample/src/ui/pages/detail/detail_page_bloc.dart';

class DetailPage extends StatefulWidget {
  static const routeName = "/detail";

  DetailPage({Key key}) : super(key: key);

  static Widget create(int postId) {
    return BlocProvider(
      bloc: DetailPageBloc(postId),
      child: DetailPage(),
    );
  }

  @override
  State createState() => _DetailPageState();
}

class _DetailPageState extends State<DetailPage> implements DetailBlocReceiver {
  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey();

  DetailPageBloc _bloc;

  @override
  void initState() {
    super.initState();
    _bloc = BlocProvider.of(context);
    _bloc.receiver = this;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _scaffoldKey,
      appBar: AppBar(
        title: Text("Post #${_bloc.post.id}"),
      ),
      body: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Card(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: <Widget>[
                  Container(
                    margin: const EdgeInsets.symmetric(vertical: 4.0),
                    child: Text("id: ${_bloc.post.id}"),
                  ),
                  Container(
                    margin: const EdgeInsets.symmetric(vertical: 4.0),
                    child: Text("userId: ${_bloc.post.userId}"),
                  ),
                  Container(
                    margin: const EdgeInsets.symmetric(vertical: 4.0),
                    child: Text("title: ${_bloc.post.title}"),
                  ),
                  Container(
                    margin: const EdgeInsets.symmetric(vertical: 4.0),
                    child: Text("body: ${_bloc.post.body}"),
                  ),
                ],
              ),
            ),
          ),
          Container(
            margin: const EdgeInsets.only(top: 8.0),
            child: Text("Comments"),
          ),
          Divider(),
          StreamBuilder(
            stream: _bloc.commentStream,
            builder:
                (BuildContext context, AsyncSnapshot<List<Comment>> snapshot) {
              if (snapshot.hasData) {
                return Expanded(
                  child: ListView(
                    children: snapshot.data
                        .map((c) => _createCommentListItem(c))
                        .toList(),
                  ),
                );
              } else {
                return Center(child: Text("No comments for this post"));
              }
            },
          )
        ],
      ),
    );
  }

  Widget _createCommentListItem(Comment comment) {
    return InkWell(
      onTap: () => _bloc.showCommentEmail(comment),
      child: Card(
        child: Padding(
          padding: const EdgeInsets.all(8.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Container(
                margin: const EdgeInsets.symmetric(vertical: 4.0),
                child: Text("${comment.name}"),
              ),
              Container(
                margin: const EdgeInsets.symmetric(vertical: 4.0),
                child: Text("${comment.body}"),
              ),
            ],
          ),
        ),
      ),
    );
  }

  @override
  void showMessage(String message) {
    _scaffoldKey.currentState.showSnackBar(SnackBar(
      content: Text(message),
    ));
  }
}
