import 'dart:convert';

import 'package:mwb/bloc_provider.dart';
import 'package:rxdart/rxdart.dart';
import 'package:sample/src/app_scope/app_bloc.dart';
import 'package:sample/src/models/comment.dart';
import 'package:sample/src/models/post.dart';

class DetailPageBloc extends BaseBloc with AppBlocProvider {
  /// Stream controller wrapper for all comments of a post.
  ///
  /// A [Subject] is basically a simplified [StreamController] with a combination
  /// of a [Stream] and a [Sink] that are linked together.
  /// In this case a [BehaviorSubject] stores the latest value received over it's sink
  /// which it also broadcasts to all listeners when they subscribe to the subjects stream.
  /// If you only want to pass values along without caching them
  /// you can use a [PublishSubject].
  final BehaviorSubject<List<Comment>> _commentsSubject =
      BehaviorSubject.seeded(null);

  Stream<List<Comment>> get commentStream => _commentsSubject.stream;

  DetailBlocReceiver receiver;
  Post post;

  DetailPageBloc(int postId) {
    post = _readPostData(postId);
  }

  @override
  Future init() async {
    _commentsSubject.sink.add(await _getPostComments());
  }

  @override
  Future dispose() async {
    await _commentsSubject.drain();
    _commentsSubject.close();
  }

  void showCommentEmail(Comment comment) {
    receiver?.showMessage(comment.email);
  }

  Post _readPostData(int id) {
    return repository.currentPosts.firstWhere((el) => el.id == id);
  }

  Future<List<Comment>> _getPostComments() async {
    final res = await repository.dataProvider.fetchPostComments(post.id);
    final json = jsonDecode(res.body);
    if (json is List<dynamic>) {
      final list = json.map((el) => Comment.fromJSON(el)).toList();
      return list;
    }
    return null;
  }
}

abstract class DetailBlocReceiver {
  void showMessage(String message);
}
