import 'package:flutter/material.dart';
import 'package:mwb/bloc_provider.dart';
import 'package:sample/src/models/post.dart';
import 'package:sample/src/ui/pages/posts/posts_page_bloc.dart';

/// Overview page of all posts that you get from the JSON placeholder site.
class PostsPage extends StatefulWidget {
  static const routeName = "/home";

  PostsPage({Key key}) : super(key: key);

  static Widget create() {
    return BlocProvider(
      bloc: PostsPageBloc(),
      child: PostsPage(),
    );
  }

  @override
  _PostsPageState createState() => _PostsPageState();
}

class _PostsPageState extends State<PostsPage> {
  PostsPageBloc _bloc;

  @override
  void initState() {
    super.initState();
    _bloc = BlocProvider.of(context);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Posts"),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.refresh),
            onPressed: _bloc.refreshList,
          )
        ],
      ),
      body: StreamBuilder(
        stream: _bloc.postsStream,
        builder: (BuildContext context, AsyncSnapshot<List<Post>> snapshot) {
          if (snapshot.hasData) {
            return ListView(
              children: snapshot.data
                  .map((post) => _createPostListItem(post))
                  .toList(),
            );
          } else {
            return Center(
              child: Text("No posts yet"),
            );
          }
        },
      ),
    );
  }

  Widget _createPostListItem(Post post) {
    return Card(
      child: InkWell(
        onTap: () => _bloc.showPostDetails(post.id),
        child: Container(
          margin: const EdgeInsets.symmetric(horizontal: 16.0, vertical: 8.0),
          child: Text(post.title),
        ),
      ),
    );
  }
}
