import 'dart:async';
import 'dart:convert';

import 'package:mwb/bloc_provider.dart';
import 'package:rxdart/rxdart.dart';
import 'package:sample/src/app_scope/app_bloc.dart';
import 'package:sample/src/models/post.dart';
import 'package:sample/src/ui/pages/detail/detail_page.dart';

class PostsPageBloc extends BaseBloc with AppBlocProvider {
  /// Stream controller wrapper for all posts.
  ///
  /// A [Subject] is basically a simplified [StreamController] with a combination
  /// of a [Stream] and a [Sink] that are linked together.
  /// In this case a [BehaviorSubject] stores the latest value received over it's sink
  /// which it also broadcasts to all listeners when they subscribe to the subjects stream.
  /// If you only want to pass values along without caching them
  /// you can use a [PublishSubject].
  final BehaviorSubject<List<Post>> _postsSubject =
      BehaviorSubject.seeded(null);

  Stream<List<Post>> get postsStream => _postsSubject.stream;

  @override
  void init() {}

  @override
  Future dispose() async {
    await _postsSubject.drain();
    _postsSubject.close();
  }

  Future<List<Post>> _getSamplePosts() async {
    final res = await repository.dataProvider.fetchPosts();
    final json = jsonDecode(res.body);
    if (json is List<dynamic>) {
      final list = json.map((el) => Post.fromJSON(el)).toList();
      return list;
    }
    return null;
  }

  Future refreshList() async {
    repository.currentPosts = await _getSamplePosts();
    _postsSubject.sink.add(repository.currentPosts);
  }

  void showPostDetails(int postId) {
    navService.pushNamed(DetailPage.routeName, args: postId);
  }
}
