import 'package:flutter/material.dart';
import 'package:sample/src/app_scope/app_bloc.dart';
import 'package:sample/src/ui/pages/posts/posts_page.dart';

/// Default page the app can either start with or fall back into.
///
/// Can be used as a landing page, where necessary initialization processes would happen that
/// need to happen before the app shows anything else.
class SplashScreenPage extends StatefulWidget {
  static const routeName = "/";

  static Widget create() => SplashScreenPage();

  @override
  State createState() => _SplashScreenState();
}

class _SplashScreenState extends State<SplashScreenPage> with AppBlocProvider {
  @override
  void initState() {
    super.initState();
    _init();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        color: Colors.white,
        child: Center(
          child: CircularProgressIndicator(),
        ),
      ),
    );
  }

  Future _init() async {
    /// Simulated async call that takes time to complete.
    ///
    /// In a real scenario, this could be the initialization of a DAO,
    /// setup of a connection to a service, or some task that takes time and
    /// simply needs to happen before the app shows anything at all.
    return Future.delayed(Duration(seconds: 1), () async {
      navService.pushReplacementNamed(PostsPage.routeName);
    });
  }
}
